//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc3_env.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Environment for test_project
//
//=============================================================================

`ifndef TEST_PROJECT_ENV_SV
`define TEST_PROJECT_ENV_SV

//-----------------------------------------------------------------------------
class test_project_env extends uvm_env;
//-----------------------------------------------------------------------------

  uvc1_env	uvc1_env_i;
  uvc2_env	uvc2_env_i;
  uvc3_env	uvc3_env_i;

  //test_project_config	cfg;

  test_project_sequencer	v_sqr;

  `uvm_component_utils(test_project_env)

  extern function new(string name = "test_project_env", uvm_component parent = null);
  extern function void build_phase(uvm_phase phase);
  extern function void connect_phase(uvm_phase phase);
  extern task run_phase(uvm_phase phase);
  extern function void check_phase(uvm_phase phase);
endclass : test_project_env 

//-----------------------------------------------------------------------------
// test_project_env implementation
//-----------------------------------------------------------------------------
function test_project_env::new(string name = "test_project_env", uvm_component parent = null);
   super.new(name,parent);
endfunction : new

function void test_project_env::build_phase(uvm_phase phase);
  super.build_phase(phase);
  uvc1_env_i = uvc1_env::type_id::create("uvc1_env_i", this);
  uvc2_env_i = uvc2_env::type_id::create("uvc2_env_i", this);
  uvc3_env_i = uvc3_env::type_id::create("uvc3_env_i", this);
  v_sqr = test_project_sequencer::type_id::create("v_sqr", this);
endfunction : build_phase

function void test_project_env::connect_phase(uvm_phase phase);

  super.connect_phase(phase);
  `uvm_info(get_type_name(), "In connect_phase", UVM_HIGH)

  // Virtual sequencer connections
  v_sqr.uvc1_sequencer_i = uvc1_env_i.agent.sequencer;
  v_sqr.uvc2_sequencer_i = uvc2_env_i.agent.sequencer;
  v_sqr.uvc3_sequencer_i = uvc3_env_i.agent.sequencer;
endfunction : connect_phase


task test_project_env::run_phase(uvm_phase phase);

  super.run_phase(phase);
  `uvm_info(get_type_name(), "In run_phase", UVM_HIGH)
  # 100ns;
endtask : run_phase

function void test_project_env::check_phase(uvm_phase phase);

  super.check_phase(phase);
  `uvm_info(get_type_name(), "In check_phase", UVM_HIGH)
endfunction : check_phase

`endif // TEST_PROJECT_ENV_SV

