//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: test_project_seq_lib.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Sequence library for test_project
//
//=============================================================================

`ifndef TEST_PROJECT_SEQ_LIB_SV
`define TEST_PROJECT_SEQ_LIB_SV

//=============================================================================
// Sequence    : uvc3_base_seq
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : Common base sequence for all sequences
//=============================================================================
//-----------------------------------------------------------------------------
class test_project_base_seq extends uvm_sequence #(uvm_sequence_item);
//-----------------------------------------------------------------------------

  test_project_config  cfg;

  `uvm_object_utils_begin(test_project_base_seq)
    `uvm_field_object(cfg, UVM_ALL_ON)
  `uvm_object_utils_end

  test_project_sequencer	v_sqr;
  uvc1_sequencer	uvc1_sequencer_i;
  uvc2_sequencer	uvc2_sequencer_i;
  uvc3_sequencer	uvc3_sequencer_i;

  extern function new(string name = "test_project_base_seq");

  extern task body();
  //---------------------------------------------------------------------------
  // constraints
  //---------------------------------------------------------------------------

endclass : test_project_base_seq

//-----------------------------------------------------------------------------
// test_project_base_seq implementation
//-----------------------------------------------------------------------------

function test_project_base_seq::new(string name = "test_project_base_seq");
   super.new(name);
endfunction : new
task test_project_base_seq::body();
  if(!$cast(v_sqr, m_sequencer)) begin
    `uvm_error(get_full_name(), "Virtual sequencer pointer cast failed");
  end
  uvc1_sequencer_i = v_sqr.uvc1_sequencer_i;
  uvc2_sequencer_i = v_sqr.uvc2_sequencer_i;
  uvc3_sequencer_i = v_sqr.uvc3_sequencer_i;
endtask: body

//=============================================================================
// Sequence    : test_project_default_seq
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : Sequence generating 1 item;
//=============================================================================
class test_project_default_seq extends test_project_base_seq;


  `uvm_object_utils(test_project_default_seq)

  // UVC sequences
  uvc1_default_seq	uvc1_seq;
  uvc2_default_seq	uvc2_seq;
  uvc3_default_seq	uvc3_seq;

  extern function new(string name = "test_project_default_seq");
  extern task body();

endclass : test_project_default_seq

//-----------------------------------------------------------------------------
// test_project_default_seq implementation
//-----------------------------------------------------------------------------

function test_project_default_seq::new(string name = "test_project_default_seq");
   super.new(name);
endfunction : new

task test_project_default_seq::body();
  super.body();
  `uvm_info(get_type_name(),"default sequence starting", UVM_MEDIUM)
    fork
      `uvm_do_on(uvc1_seq,  uvc1_sequencer_i)
      `uvm_do_on(uvc2_seq,  uvc2_sequencer_i)
      `uvm_do_on(uvc3_seq,  uvc3_sequencer_i)
    join
   `uvm_info(get_type_name(),"default sequence completed",UVM_MEDIUM)
endtask : body

//additional sequences can be included in this file

`endif // TEST_PROJECT_SEQ_LIB_SV

