//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: test_project_base_test.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Base test packagetest_project:
//
//=============================================================================

`ifndef TEST_PROJECT_BASE_TEST_SV
`define TEST_PROJECT_BASE_TEST_SV

//-----------------------------------------------------------------------------
class test_project_base_test extends uvm_test;
//-----------------------------------------------------------------------------

  uvc1_config	uvc1_cfg;
  uvc2_config	uvc2_cfg;
  uvc3_config	uvc3_cfg;

  test_project_config cfg;

  test_project_env    env;

  `uvm_component_utils(test_project_base_test)
//---------------------------------------------------------------------------
// methods
//---------------------------------------------------------------------------

  extern function new(string name = "test_project_base_test", uvm_component parent = null);
  extern function void build_phase(uvm_phase phase);
  extern function void start_of_simulation_phase(uvm_phase phase);
//---------------------------------------------------------------------------
// constraints
//---------------------------------------------------------------------------

endclass : test_project_base_test
//---------------------------------------------------------------------------
// Implementation
//---------------------------------------------------------------------------

function test_project_base_test::new(string name = "test_project_base_test", uvm_component parent = null);
  super.new(name, parent);
endfunction : new

function void test_project_base_test::build_phase(uvm_phase phase);
  super.build_phase(phase);
  uvc1_cfg = uvc1_config::type_id::create("uvc1_cfg");
  uvc2_cfg = uvc2_config::type_id::create("uvc2_cfg");
  uvc3_cfg = uvc3_config::type_id::create("uvc3_cfg");
  cfg = test_project_config::type_id::create("cfg");

  // Configure env
  if(!uvm_config_db #(virtual uvc1_if)::get(this, "", "uvc1_if_i", uvc1_cfg.vif))
    `uvm_error(get_type_name(), "no virtual interface found")
  if(!uvm_config_db #(virtual uvc2_if)::get(this, "", "uvc2_if_i", uvc2_cfg.vif))
    `uvm_error(get_type_name(), "no virtual interface found")
  if(!uvm_config_db #(virtual uvc3_if)::get(this, "", "uvc3_if_i", uvc3_cfg.vif))
    `uvm_error(get_type_name(), "no virtual interface found")

  uvm_config_db #(uvc1_config)::set(this, "*", "uvc1_config", uvc1_cfg);
  uvm_config_db #(uvc2_config)::set(this, "*", "uvc2_config", uvc2_cfg);
  uvm_config_db #(uvc3_config)::set(this, "*", "uvc3_config", uvc3_cfg);

  cfg.uvc1_cfg = uvc1_cfg;
  cfg.uvc2_cfg = uvc2_cfg;
  cfg.uvc3_cfg = uvc3_cfg;

  uvm_config_db #(int)::set(this, "env.uvc1_env_i.agent.sequencer", "count", 0);
  uvm_config_db #(int)::set(this, "env.uvc1_env_i.agent", "is_active", UVM_ACTIVE);
  uvm_config_db #(int)::set(this, "env.uvc2_env_i.agent.sequencer", "count", 0);
  uvm_config_db #(int)::set(this, "env.uvc2_env_i.agent", "is_active", UVM_ACTIVE);
  uvm_config_db #(int)::set(this, "env.uvc3_env_i.agent.sequencer", "count", 0);
  uvm_config_db #(int)::set(this, "env.uvc3_env_i.agent", "is_active", UVM_ACTIVE);

  // Create env
  env = test_project_env::type_id::create("env", this);
endfunction : build_phase

function void test_project_base_test::start_of_simulation_phase(uvm_phase phase);
  super.start_of_simulation_phase(phase);
  `uvm_info(get_type_name(), $psprintf("Verbosity level is set to: %d", get_report_verbosity_level()), UVM_MEDIUM);
  uvm_top.print_topology();
  `uvm_info(get_type_name(), "Print all Factory overrides", UVM_HIGH);
  factory.print();
  uvm_top.set_report_id_action_hier("ILLEGALNAME", UVM_NO_ACTION);
endfunction : start_of_simulation_phase

`endif // TEST_PROJECT_BASE_TEST_SV

