//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc1_agent.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Agent for uvc1
//
//=============================================================================

`ifndef UVC1_AGENT_SV
`define UVC1_AGENT_SV

//-----------------------------------------------------------------------------
class uvc1_agent extends uvm_agent;
//-----------------------------------------------------------------------------

  uvm_active_passive_enum is_active = UVM_ACTIVE;
  uvc1_sequencer sequencer;
  uvc1_driver driver;
  uvc1_monitor monitor;
  uvc1_coverage coverage;

  `uvm_component_utils_begin(uvc1_agent)
    `uvm_field_enum(uvm_active_passive_enum, is_active, UVM_ALL_ON)
  `uvm_component_utils_end

  extern function new(string name = "uvc1_agent", uvm_component parent);
  extern function void build_phase(uvm_phase phase);
  extern function void connect_phase(uvm_phase phase);

endclass : uvc1_agent 

function uvc1_agent::new(string name = "uvc1_agent", uvm_component parent);
   super.new(name,parent);
endfunction : new

function void uvc1_agent::build_phase(uvm_phase phase);
  super.build_phase(phase);
  `uvm_info(get_type_name(), $psprintf("agent is: %s", is_active), UVM_MEDIUM)
  monitor = uvc1_monitor::type_id::create("monitor",this);
  coverage= uvc1_coverage::type_id::create("coverage",this);
  if (is_active == UVM_ACTIVE)
    begin
      driver    = uvc1_driver::type_id::create("driver",this);
      sequencer = uvc1_sequencer::type_id::create("sequencer",this);
    end
endfunction : build_phase

function void uvc1_agent::connect_phase(uvm_phase phase);
  super.connect_phase(phase);
  monitor.ap.connect(coverage.analysis_export);
  if (is_active == UVM_ACTIVE)
    begin
      driver.seq_item_port.connect(sequencer.seq_item_export);
    end
endfunction : connect_phase

`endif // UVC1_AGENT_SV

