//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc1_monitor.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Monitor for uvc1
//
//=============================================================================

`ifndef UVC1_MONITOR_SV
`define UVC1_MONITOR_SV

//-----------------------------------------------------------------------------
class uvc1_monitor extends uvm_monitor;
//-----------------------------------------------------------------------------

virtual uvc1_if vif;

uvc1_seq_item trans;

uvm_analysis_port #(uvc1_seq_item) ap;

  uvc1_config  cfg;

`uvm_component_utils_begin(uvc1_monitor)
  `uvm_field_object(trans, UVM_ALL_ON)
`uvm_component_utils_end

  extern function new(string name = "uvc1_monitor", uvm_component parent = null);
  extern function void build_phase(uvm_phase phase);
  extern function void connect_phase(uvm_phase phase);
  extern task run_phase(uvm_phase phase);
  extern task collect_data();
endclass : uvc1_monitor 

//-----------------------------------------------------------------------------
// uvc1_monitor implementation
//-----------------------------------------------------------------------------
function uvc1_monitor::new(string name = "uvc1_monitor", uvm_component parent = null);
  super.new(name,parent);
endfunction : new

function void uvc1_monitor::build_phase(uvm_phase phase);
  super.build_phase(phase);
  ap = new("ap", this);
endfunction : build_phase

function void uvc1_monitor::connect_phase(uvm_phase phase);
  super.connect_phase(phase);
  `uvm_info(get_type_name(),"connect phase",UVM_MEDIUM)
  if (!uvm_config_db #(uvc1_config)::get(this, "", "uvc1_config" , cfg))
  `uvm_error(get_type_name(), "Config Error uvm_config_db #(uvc1_config)::get cannot find resource cfg")
  vif = cfg.vif;
endfunction : connect_phase

task uvc1_monitor::run_phase(uvm_phase phase);
  `uvm_info(get_type_name(),"RUNNING:",UVM_MEDIUM)
  collect_data();
endtask : run_phase

task uvc1_monitor::collect_data();
  trans = new();
  forever
    begin
      //put code to collect bus transactions here
      #10 ;

    ap.write(trans);
    end;
endtask : collect_data

`endif // UVC1_MONITOR_SV

