//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc1_seq_lib.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Sequence library for uvc1
//
//=============================================================================

`ifndef UVC1_SEQ_LIB_SV
`define UVC1_SEQ_LIB_SV

//=============================================================================
// Sequence    : uvc1_base_seq
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : Common base sequence for all sequences
//=============================================================================
//-----------------------------------------------------------------------------
class uvc1_base_seq extends uvm_sequence #(uvc1_seq_item);
//-----------------------------------------------------------------------------

  uvc1_config  cfg;
  uvc1_seq_item  item;
  `uvm_object_utils_begin(uvc1_base_seq)
    `uvm_field_object(item, UVM_ALL_ON)
    `uvm_field_object(cfg, UVM_ALL_ON)
  `uvm_object_utils_end

  extern function new(string name = "uvc1_base_seq");
endclass : uvc1_base_seq

//-----------------------------------------------------------------------------
// uvc1_base_seq implementation
//-----------------------------------------------------------------------------

function uvc1_base_seq::new(string name = "uvc1_base_seq");
   super.new(name);
endfunction : new

//=============================================================================
// Sequence    : uvc1_default_seq
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : Sequence generating 1 item;
//=============================================================================
class uvc1_default_seq extends uvc1_base_seq;

`uvm_object_utils(uvc1_default_seq)

  extern function new(string name = "uvc1_default_seq");
  extern task body();

endclass : uvc1_default_seq

//-----------------------------------------------------------------------------
// uvc1_default_seq implementation
//-----------------------------------------------------------------------------

function uvc1_default_seq::new(string name = "uvc1_default_seq");
   super.new(name);
endfunction : new

task uvc1_default_seq::body();
  `uvm_info(get_type_name(),"default sequence starting", UVM_MEDIUM)
   repeat(5) 
    `uvm_do(item)               //this line sends the transaction
  `uvm_info(get_type_name(),"default sequence completed", UVM_MEDIUM)
endtask : body

//=============================================================================
// Sequence    : uvc1_seq1
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : seq1
//=============================================================================
class uvc1_seq1 extends uvc1_base_seq;

rand int delay1;
rand int delay2;
constraint d1 {delay1 inside {[55:125]};}
constraint d2 {delay2 inside {[25:45]};}

`uvm_object_utils_begin(uvc1_seq1)
   `uvm_field_int(delay1,UVM_ALL_ON + UVM_DEC)
   `uvm_field_int(delay2,UVM_ALL_ON + UVM_DEC)
`uvm_object_utils_end

  extern function new(string name = "uvc1_seq1");
  extern task body();

endclass : uvc1_seq1 

//-----------------------------------------------------------------------------
// uvc1_seq1 implementation
//-----------------------------------------------------------------------------

function uvc1_seq1::new(string name = "uvc1_seq1");
   super.new(name);
endfunction : new

task uvc1_seq1::body();
  `uvm_info(get_type_name(),"sequence RUNNING",UVM_MEDIUM)
  //not the best way to print, just to confirm randomization
  `uvm_info(get_type_name(),$sformatf("delay1=%0d, delay2=%0d",delay1, delay2),UVM_HIGH)
  #delay1;  //dummy delay to illustrate a sequence rand variable
  `uvm_do(item) //this line sends the transaction
  #delay1;
  `uvm_do(item) //this line sends the transaction
  #delay1;
  `uvm_do(item) //this line sends the transaction
  #delay2;
  `uvm_info(get_type_name(),"sequence COMPLETE",UVM_MEDIUM)
endtask : body

//=============================================================================
// Sequence    : uvc1_reset_seq
// Author      : <author>    Tel: <phone>   Email: <email>
// Using       : <list of sub-sequences used by this sequences>
// Description : reset_seq
//=============================================================================
class uvc1_reset_seq  extends uvc1_base_seq;
//"req" built-in uvm_sequence class member for sequence_item
`uvm_object_utils(uvc1_reset_seq)

  extern function new(string name = "uvc1_reset_seq");
  extern task body();
endclass : uvc1_reset_seq 

//-----------------------------------------------------------------------------
// uvc1_reset_seq implementation
//-----------------------------------------------------------------------------

function uvc1_reset_seq::new(string name = "uvc1_reset_seq");
   super.new(name);
endfunction : new

task uvc1_reset_seq::body();
  `uvm_info(get_type_name(),"RUNNING a dummy reset sequence in the reset phase",UVM_MEDIUM)
  `uvm_info(get_type_name(),"  sequence is transmitted in the regular driver",UVM_MEDIUM)
   #5;
   `uvm_do(item) //send dummy reset sequence 
   #5;
  `uvm_info(get_type_name(),"sequence COMPLETE",UVM_MEDIUM)
endtask : body
//additional sequences can be included in this file

`endif // UVC1_SEQ_LIB_SV

