//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc3_agent.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Agent for uvc3
//
//=============================================================================

`ifndef UVC3_AGENT_SV
`define UVC3_AGENT_SV

//-----------------------------------------------------------------------------
class uvc3_agent extends uvm_agent;
//-----------------------------------------------------------------------------

  uvm_active_passive_enum is_active = UVM_ACTIVE;
  uvc3_sequencer sequencer;
  uvc3_driver driver;
  uvc3_monitor monitor;
  uvc3_coverage coverage;

  `uvm_component_utils_begin(uvc3_agent)
    `uvm_field_enum(uvm_active_passive_enum, is_active, UVM_ALL_ON)
  `uvm_component_utils_end

  extern function new(string name = "uvc3_agent", uvm_component parent);
  extern function void build_phase(uvm_phase phase);
  extern function void connect_phase(uvm_phase phase);

endclass : uvc3_agent 

function uvc3_agent::new(string name = "uvc3_agent", uvm_component parent);
   super.new(name,parent);
endfunction : new

function void uvc3_agent::build_phase(uvm_phase phase);
  super.build_phase(phase);
  `uvm_info(get_type_name(), $psprintf("agent is: %s", is_active), UVM_MEDIUM)
  monitor = uvc3_monitor::type_id::create("monitor",this);
  coverage= uvc3_coverage::type_id::create("coverage",this);
  if (is_active == UVM_ACTIVE)
    begin
      driver    = uvc3_driver::type_id::create("driver",this);
      sequencer = uvc3_sequencer::type_id::create("sequencer",this);
    end
endfunction : build_phase

function void uvc3_agent::connect_phase(uvm_phase phase);
  super.connect_phase(phase);
  monitor.ap.connect(coverage.analysis_export);
  if (is_active == UVM_ACTIVE)
    begin
      driver.seq_item_port.connect(sequencer.seq_item_export);
    end
endfunction : connect_phase

`endif // UVC3_AGENT_SV

