//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc3_driver.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Driver for uvc3
//
//=============================================================================

`ifndef UVC3_DRIVER_SV
`define UVC3_DRIVER_SV

//-----------------------------------------------------------------------------
class uvc3_driver extends uvm_driver #(uvc3_seq_item);
//-----------------------------------------------------------------------------

  virtual uvc3_if vif;
  uvc3_config  cfg;
  uvc3_seq_item         item;

  `uvm_component_utils_begin(uvc3_driver)
    `uvm_field_object(item, UVM_ALL_ON)
  `uvm_component_utils_end

  extern function new(string name = "uvc3_driver", uvm_component parent = null);
  extern function void connect_phase(uvm_phase phase);
  extern task run_phase(uvm_phase phase);
  extern task get_and_drive(uvm_phase phase);
  extern task send_to_dut(input uvc3_seq_item  item);
  extern function void report_phase(uvm_phase phase);
endclass : uvc3_driver 

//-----------------------------------------------------------------------------
// uvc3_driver implementation
//-----------------------------------------------------------------------------
function uvc3_driver::new(string name = "uvc3_driver", uvm_component parent = null);
  super.new(name,parent);
endfunction : new

function void uvc3_driver::connect_phase(uvm_phase phase);
  super.connect_phase(phase);
  `uvm_info(get_type_name(),"connect phase",UVM_MEDIUM)
  if (!uvm_config_db #(uvc3_config)::get(this, "", "uvc3_config" , cfg))
  `uvm_error(get_type_name(), "Config Error uvm_config_db #(uvc3_config)::get cannot find resource cfg")
  vif = cfg.vif;
endfunction : connect_phase

task uvc3_driver::run_phase(uvm_phase phase);
  super.run_phase(phase);
  `uvm_info(get_type_name(),"RUNNING:",UVM_MEDIUM)
  get_and_drive(phase);
endtask : run_phase

task uvc3_driver::get_and_drive(uvm_phase phase);
  `uvm_info(get_type_name(),"get_and_drive",UVM_MEDIUM)
  forever begin
    seq_item_port.get_next_item(item);
    @(posedge vif.clock)
    `uvm_info(get_type_name(), "got item", UVM_MEDIUM)
    `uvm_info(get_type_name(), {"\n",item.sprint}, UVM_MEDIUM)
    send_to_dut(item);
    seq_item_port.item_done();
  end
endtask : get_and_drive

task uvc3_driver::send_to_dut(input uvc3_seq_item  item);
  //Send data to DUT (BFM - fill in your BFM code here)
  #10ns ;
endtask : send_to_dut

function void uvc3_driver::report_phase(uvm_phase phase);
  `uvm_info(get_type_name(), "Reached end of stimulus", UVM_HIGH);
  //fill in any reporting code if needed
endfunction : report_phase

`endif // UVC3_DRIVER_SV

