//=============================================================================
// Copyright(c) Dr. Christoph Suehnel 2014 confidential, all rights reserved
//=============================================================================
// Project  : test_project
//
// File Name: uvc3_monitor.sv
//
// Author   : Name   : Christoph Suehnel
//            Email  : christoph@christoph-suehnel.de
//            Tel    : +49 8822 835 9239
//            Company: Dr. Christoph Suehnel
//            Dept   : Consulting

// Version:

//=============================================================================
// Description:
//
// Monitor for uvc3
//
//=============================================================================

`ifndef UVC3_MONITOR_SV
`define UVC3_MONITOR_SV

//-----------------------------------------------------------------------------
class uvc3_monitor extends uvm_monitor;
//-----------------------------------------------------------------------------

virtual uvc3_if vif;

uvc3_seq_item trans;

uvm_analysis_port #(uvc3_seq_item) ap;

  uvc3_config  cfg;

`uvm_component_utils_begin(uvc3_monitor)
  `uvm_field_object(trans, UVM_ALL_ON)
`uvm_component_utils_end

  extern function new(string name = "uvc3_monitor", uvm_component parent = null);
  extern function void build_phase(uvm_phase phase);
  extern function void connect_phase(uvm_phase phase);
  extern task run_phase(uvm_phase phase);
  extern task collect_data();
endclass : uvc3_monitor 

//-----------------------------------------------------------------------------
// uvc3_monitor implementation
//-----------------------------------------------------------------------------
function uvc3_monitor::new(string name = "uvc3_monitor", uvm_component parent = null);
  super.new(name,parent);
endfunction : new

function void uvc3_monitor::build_phase(uvm_phase phase);
  super.build_phase(phase);
  ap = new("ap", this);
endfunction : build_phase

function void uvc3_monitor::connect_phase(uvm_phase phase);
  super.connect_phase(phase);
  `uvm_info(get_type_name(),"connect phase",UVM_MEDIUM)
  if (!uvm_config_db #(uvc3_config)::get(this, "", "uvc3_config" , cfg))
  `uvm_error(get_type_name(), "Config Error uvm_config_db #(uvc3_config)::get cannot find resource cfg")
  vif = cfg.vif;
endfunction : connect_phase

task uvc3_monitor::run_phase(uvm_phase phase);
  `uvm_info(get_type_name(),"RUNNING:",UVM_MEDIUM)
  collect_data();
endtask : run_phase

task uvc3_monitor::collect_data();
  trans = new();
  forever
    begin
      //put code to collect bus transactions here
      #10 ;

    ap.write(trans);
    end;
endtask : collect_data

`endif // UVC3_MONITOR_SV

